<?php
$guestbook_fp = NULL;
$MAX_BAD_WORD_LENGTH = 255;
$MIN_COMMENTS_LENGTH = 10;
$PREVENT_URLS_IN_COMMENTS = TRUE;
$ERROR_MSG_URLS_NOT_ALLOWED = "Error,url are not allowed in comments.";
$ENABLE_EMAIL_FIELD = TRUE;
$ENABLE_URL_FIELD = TRUE;
$ENABLE_COMMENT_FIELD = TRUE;
$MIN_SECONDS_BETWEEN_POSTS = 120;
$ERROR_MSG_FLOOD_DETECTED = "Error,you are attempting to post too frequently.";
$READ_ONLY_MODE = FALSE;
$MAX_WORD_LENGTH = 40;
$ERROR_MSG_MAX_WORD_LENGTH = "Error,you attempted to use a word that was too long.";
###########
function guestbook_file_path()
{
global $DATA_FOLDER;
return dirname(__FILE__) . '/../' . $DATA_FOLDER . "/" . "guestbook.txt";
}
###########
function guestbook_open_for_read()
{
global $guestbook_fp;
$guestbook_fp = @fopen(guestbook_file_path(), "rb");
if($guestbook_fp !== FALSE)
{
if(@flock($guestbook_fp, LOCK_SH) === FALSE)
{
guestbook_close();
return FALSE;
}
}
return $guestbook_fp;
}
##########
function guestbook_open_for_writing()
{
global $guestbook_fp;
$guestbook_fp = @fopen(guestbook_file_path(), "r+b");
if($guestbook_fp !== FALSE)
{
if(@flock($guestbook_fp, LOCK_EX) === TRUE)
{
if(@ftruncate($guestbook_fp, 0) === FALSE)
{
guestbook_close();
return FALSE;
}
}else{
guestbook_close();
return FALSE;
}
}
return $guestbook_fp;
}
##########
function guestbook_forward($forward_count)
{
global $guestbook_fp;
$count = 0;
while($count < $forward_count && !feof($guestbook_fp))
{
fgets($guestbook_fp);
$count += 1;
}
}
##########
function guestbook_next_id()
{
if(guestbook_open_for_read() === FALSE)
{
return 0;
}else{
$entry = guestbook_next();
guestbook_close();
if($entry === FALSE)
{
return 0;
}else{
return ((int)$entry['id']) + 1;
}
}
}
##########
function entry_explode($raw_entry)
{
return explode('|', trim($raw_entry));
}
##########
function guestbook_next()
{
global $guestbook_fp;
$entry_raw = fgets($guestbook_fp);
if(feof($guestbook_fp))
{
return FALSE;
}else if(empty($entry_raw))
{
return FALSE;
}else{
$entry_components = entry_explode($entry_raw);
$entry_components = array_map('rawurldecode', $entry_components);
$entry_components = array_map('htmlspecialchars', $entry_components);
$entry = @Array("id" => $entry_components[0], "name" => $entry_components[1], "email" => $entry_components[2], "url" => $entry_components[3], "comments" => $entry_components[4], "timestamp" => $entry_components[5], "ipaddress" => $entry_components[6]);
return $entry;
}
}
##########
function getIdToIdxMap($entries)
{
$id_to_idx = @Array();
$ent_idx = 0;
foreach ($entries as $entry)
{
$id_to_idx[ $entry[0] ] = $ent_idx;
$ent_idx++;
}
return $id_to_idx;
}
##########
function guestbook_delete_entries($idArray, $banip = FALSE)
{
global $dbs_error;
global $guestbook_fp;
if(guestbook_open_for_read() === FALSE)
{
die("Error,unable to open guestbook file for reading.");
}
$raw_entries = @file(guestbook_file_path());
guestbook_close();
if($raw_entries === FALSE)
{
die("Error,unable to get entries from guestbook.");
}
$entries = array_map('entry_explode', $raw_entries);
$id_to_idx = getIdToIdxMap($entries);
foreach ($idArray as $id)
{
if(!isset($id_to_idx[$id]))
{
die("Error,invalid entry id.");
}
$idx = $id_to_idx[$id];
if($idx === 0 || !empty($idx))
{
if($banip)
{
$entry_components = $entries[$idx];
$entry_components = array_map('rawurldecode', $entry_components);
if(isset($entry_components[6]))
{
$ipAddress = $entry_components[6];
if(!empty($ipAddress) && !is_banned($ipAddress))
{
ban_add($ipAddress);
}
}
}
unset($raw_entries[$idx]);
}
}
$raw_entries_flat = implode("", $raw_entries);
unset($raw_entries);
if(guestbook_open_for_writing() === FALSE)
{
die("Error,unable to open guestbook file for writing.");
}
fputs($guestbook_fp, $raw_entries_flat);
unset($raw_entries_flat);
guestbook_close();
set_guestbook_entries_count(guestbook_count_entries());
}
##########
function guestbook_validate($entry)
{
global $MAX_NAME_LENGTH;
global $MAX_EMAIL_LENGTH;
global $MAX_URL_LENGTH;
global $MAX_COMMENTS_LENGTH;
global $MIN_COMMENTS_LENGTH;
global $NAME_FIELD_NAME;
global $EMAIL_FIELD_NAME;
global $URL_FIELD_NAME;
global $COMMENTS_FIELD_NAME;
global $ERROR_MSG_BAD_WORD;
global $dbs_error;
$dbs_error = "";
validate_notempty($entry, "name", $NAME_FIELD_NAME);
global $ENABLE_COMMENT_FIELD;
if($ENABLE_COMMENT_FIELD === TRUE) 
{
if(validate_notempty($entry, "comments", $COMMENTS_FIELD_NAME)) 
{
validate_minlength($entry, "comments", $MIN_COMMENTS_LENGTH, $COMMENTS_FIELD_NAME);
}
}else{
if(isset($entry["comments"])) 
{
die("Error,comments field is disabled.");
}
}
validate_length($entry, "name", $MAX_NAME_LENGTH, $NAME_FIELD_NAME);
validate_notags($entry, "name", $NAME_FIELD_NAME);
validate_length($entry, "email", $MAX_EMAIL_LENGTH, $EMAIL_FIELD_NAME);
validate_email($entry, "email", $EMAIL_FIELD_NAME);
validate_length($entry, "url", $MAX_URL_LENGTH, $URL_FIELD_NAME);
validate_url($entry, "url", $URL_FIELD_NAME);
validate_length($entry, "comments", $MAX_COMMENTS_LENGTH, $COMMENTS_FIELD_NAME);
validate_notags($entry, "comments", $COMMENTS_FIELD_NAME);
validate_max_word_length($entry, "comments", $COMMENTS_FIELD_NAME);
if( (isset($entry["name"]) && has_bad_word($entry["name"])) || (isset($entry["comments"]) && has_bad_word($entry["comments"]))) 
{
$dbs_error .= htmlspecialchars($ERROR_MSG_BAD_WORD) . '<br/>';
}
global $PREVENT_URLS_IN_COMMENTS;
if($PREVENT_URLS_IN_COMMENTS === TRUE && isset($entry["comments"]) && has_url($entry["comments"]))
{
global $ERROR_MSG_URLS_NOT_ALLOWED;
$dbs_error .= htmlspecialchars($ERROR_MSG_URLS_NOT_ALLOWED) . '<br/>';
}
global $CHALLENGE_ENABLED;
if($CHALLENGE_ENABLED === TRUE) 
{
global $CHALLENGE_FIELD_PARAM_NAME;
$entered_challenge_value = $entry[$CHALLENGE_FIELD_PARAM_NAME];
if(!isChallengeAccepted($entered_challenge_value)) 
{
global $ERROR_MSG_BAD_CHALLENGE_STRING;
$dbs_error .= htmlspecialchars($ERROR_MSG_BAD_CHALLENGE_STRING) . '<br/>';
}
}
return empty($dbs_error);
}
##########
function guestbook_add($entry) 
{
global $READ_ONLY_MODE;
global $dbs_error;
if(guestbook_validate($entry)) 
{
if($READ_ONLY_MODE === TRUE) 
{
$dbs_error = htmlspecialchars("Error,this guestbook is in read mode.");
return FALSE;
}
$now = gmstrftime( time() );
$ipaddress = $_SERVER['REMOTE_ADDR'];
if( is_flood_detected($ipaddress) ) 
{
global $ERROR_MSG_FLOOD_DETECTED;
$dbs_error = htmlspecialchars($ERROR_MSG_FLOOD_DETECTED);
return FALSE;
}
$entry_stripped = array_map("strip_tags", $entry);
$entry_encoded = array_map("rawurlencode", $entry_stripped);
if(!file_exists(guestbook_file_path())) 
{
if(touch(guestbook_file_path()) === FALSE) 
{
$dbs_error = htmlspecialchars("Error,unable to create guestbook file in data folder.");
return FALSE;
}
}
if(guestbook_open_for_read() === FALSE) 
{
$dbs_error = htmlspecialchars("Error,unable to open guestbook file for reading.");
return FALSE;
}
$oldContents = @file_get_contents(guestbook_file_path());
guestbook_close(); 
if($oldContents === FALSE)
{
$dbs_error = htmlspecialchars("Error,Unable to get guestbook file contents.");
return FALSE;
}
$nextId = guestbook_next_id();
if(guestbook_open_for_writing() === FALSE) 
{
$dbs_error = htmlspecialchars("Error,unable to open guestbook file for writing.");
return FALSE;
}
global $guestbook_fp;
fputs($guestbook_fp,
$nextId . "|" .
value_or_blank($entry_encoded, 'name') . "|" .
value_or_blank($entry_encoded, 'email') . "|" .
value_or_blank($entry_encoded, 'url') . "|" .
value_or_blank($entry_encoded, 'comments') . "|" .
$now . "|" .
$ipaddress . "\n"
);
fputs($guestbook_fp, $oldContents);
unset($oldContents);
guestbook_close();
set_guestbook_entries_count(guestbook_count_entries());
global $ADMIN_EMAIL_ADDRESS;
if(isset($ADMIN_EMAIL_ADDRESS) && !empty($ADMIN_EMAIL_ADDRESS)) 
{
$headers = "From:  " . value_or_blank($entry_stripped, 'email') . "\r\n" . "Reply-To:  " . value_or_blank($entry_stripped, 'email') . "\r\n" . "X-Mailer: PHP/" . phpversion();
if(mail($ADMIN_EMAIL_ADDRESS, "Guestbook message from " . value_or_blank($entry_stripped, 'name'), "
" . value_or_blank($entry_stripped, 'name') . "\n" . "
" . value_or_blank($entry_stripped, 'url') . "\n" . "
" . value_or_blank($entry_stripped, 'comments'), $headers) !== TRUE) 
{
$dbs_error = htmlspecialchars("Error,unable to send notification.");
return FALSE;
}
}
return TRUE;
}
return FALSE;
}
##########
function guestbook_close() 
{
global $guestbook_fp;
@fclose($guestbook_fp);
}
##########
function ban_file_path() 
{
global $DATA_FOLDER;
return dirname(__FILE__) . '/../' . $DATA_FOLDER . "/" . "bans.txt";
}
##########
function ban_list() 
{
$banlist = @file(ban_file_path());
if($banlist !== FALSE) 
{
$banlist = array_map("trim", $banlist);
}
return $banlist;
}
##########
function is_banned($ipaddress) 
{
$bans = ban_list();
if($bans !== FALSE) 
{
return( array_search(trim($ipaddress), $bans) !== FALSE );
}else{
return FALSE;
}
}
##########
function ban_add($ipaddress) 
{
$ban_fp = @fopen(ban_file_path(), "a");
if($ban_fp === FALSE) 
{
die("Error,unable to open ban file for writing.");
}
@flock($ban_fp, LOCK_EX);
fputs($ban_fp, $ipaddress . "\n");
@fclose($ban_fp);
}
##########
function unban($ipAddressArray) 
{
$bans = ban_list();
if($bans !== FALSE) 
{
foreach ($ipAddressArray as $ipaddress) 
{
$idx = array_search(trim($ipaddress), $bans);
if($idx !== FALSE)
{
unset($bans[$idx]);
}else{
die("Error,an invalid ip address was specified.");
}
}
$raw_bans_flat = implode("\n", $bans);
if(!empty($raw_bans_flat)) $raw_bans_flat .= "\n";
$ban_fp = @fopen(ban_file_path(), "w");
if($ban_fp === FALSE) 
{
die("Error,unable to open ban file for writing.");
}
@flock($ban_fp, LOCK_EX);
fputs($ban_fp, $raw_bans_flat);
@fclose($ban_fp);
}else{
die("Error,unable to get list of current bans.");
}
}
##########
function bad_word_file_path() 
{
global $DATA_FOLDER;
return dirname(__FILE__) . '/../' . $DATA_FOLDER . "/" . "bad_words.txt";
}
##########
function bad_word_list() 
{
$bad_word_list = @file(bad_word_file_path());
if($bad_word_list !== FALSE) 
{
$bad_word_list = array_map("trim", $bad_word_list);
}
return $bad_word_list;
}
##########
function is_bad_word($word) 
{
$bad_words = bad_word_list();
if($bad_words !== FALSE) 
{
return( array_casesearch(trim($word), $bad_words) !== FALSE );
}else{
return FALSE;
}
}
##########
function has_url($text) 
{
if(preg_match('/http:|https:|ftp:/i', $text)) 
{
return TRUE;
} else {
return FALSE;
}
}
##########
function has_bad_word($text) 
{
$bad_words = bad_word_list();
if($bad_words !== FALSE) 
{
foreach ($bad_words as $bad_word) 
{
if(preg_match('/(\b|[^A-Za-z0-9])' . $bad_word . '(\b|[^A-Za-z0-9])/i', $text)) 
{
return TRUE;
}
}
}
return FALSE;
}
##########
function bad_word_add($word) 
{
global $MAX_BAD_WORD_LENGTH;
if($word === NULL || strlen($word) === 0) 
{
return FALSE;
}
if(strlen($word) > $MAX_BAD_WORD_LENGTH) 
{
die("Error,that word is too long.");
}
if(is_bad_word($word)) 
{
return FALSE;
}
$bad_word_fp = @fopen(bad_word_file_path(), "a");
if($bad_word_fp === FALSE)
{
die("Error,unable to open bad word file for writing.");
}
@flock($bad_word_fp, LOCK_EX);
fputs($bad_word_fp, $word . "\n");
@fclose($bad_word_fp);
}
##########
function remove_bad_word($wordArray)
{
$bad_words = bad_word_list();
if($bad_words !== FALSE)
{
foreach ($wordArray as $word)
{
$idx = array_search(trim($word), $bad_words);
if($idx !== FALSE) {
unset($bad_words[$idx]);
}else{
die("Error,an invalid bad word was specified.");
}
}
$raw_bad_words_flat = implode("\n", $bad_words);
if(!empty($raw_bad_words_flat)) $raw_bad_words_flat .= "\n";
$bad_word_fp = @fopen(bad_word_file_path(), "w");
if($bad_word_fp === FALSE)
{
die("Error,unable to open bad word file for writing.");
}
@flock($bad_word_fp, LOCK_EX);
fputs($bad_word_fp, $raw_bad_words_flat);
@fclose($bad_word_fp);
}else{
die("Error,unable to get list of current bad words.");
}
}
##########
function is_flood_detected($ipaddress)
{
global $MIN_SECONDS_BETWEEN_POSTS;
if($MIN_SECONDS_BETWEEN_POSTS <= 0) return FALSE;
$timestamp_threshold = time() - $MIN_SECONDS_BETWEEN_POSTS;
$guestbookExists = (guestbook_open_for_read() !== FALSE);
if($guestbookExists)
{
while( ($entry = guestbook_next()) !== FALSE &&
intval($entry["timestamp"]) >= $timestamp_threshold)
{
if($entry["ipaddress"] === $ipaddress)
{
guestbook_close();
return TRUE;
}
}
guestbook_close();
}
return FALSE;
}
##########
function guestbook_summary_file_path()
{
global $DATA_FOLDER;
return dirname(__FILE__) . '/../' . $DATA_FOLDER . "/" . "guestbook_summary.txt";
}
##########
function guestbook_count_entries()
{
if(guestbook_open_for_read() === FALSE)
{
return 0;
}
$raw_entries = @file(guestbook_file_path());
guestbook_close();
if($raw_entries === FALSE)
{
return 0;
}
return sizeof($raw_entries);
}
$guestbook_entries_count = -1;
##########
function get_guestbook_entries_count()
{
global $guestbook_entries_count;
if($guestbook_entries_count >= 0)
{
return $guestbook_entries_count;
}
$summarylist = @file(guestbook_summary_file_path());
if($summarylist !== FALSE)
{
$guestbook_entries_count = intval($summarylist[0]);
return $guestbook_entries_count;
}else{
$count = guestbook_count_entries();
set_guestbook_entries_count($count);
return $count;
}
}
##########
function set_guestbook_entries_count($count)
{
$summary_fp = @fopen(guestbook_summary_file_path(), "w");
if($summary_fp === FALSE)
{
die("Error,unable to open summary file for writing.");
}
@flock($summary_fp, LOCK_EX);
fputs($summary_fp, $count . "\n");
fclose($summary_fp);
global $guestbook_entries_count;
$guestbook_entries_count = $count;
}
?>